# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'qhangups/qhangupsconversationwidget.ui'
#
# Created: Wed May 20 12:38:45 2015
#      by: PyQt4 UI code generator 4.11.3
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)

class Ui_QHangupsConversationWidget(object):
    def setupUi(self, QHangupsConversationWidget):
        QHangupsConversationWidget.setObjectName(_fromUtf8("QHangupsConversationWidget"))
        QHangupsConversationWidget.resize(500, 350)
        self.verticalLayout_2 = QtGui.QVBoxLayout(QHangupsConversationWidget)
        self.verticalLayout_2.setObjectName(_fromUtf8("verticalLayout_2"))
        self.messagesFrame = QtGui.QFrame(QHangupsConversationWidget)
        self.messagesFrame.setFrameShape(QtGui.QFrame.StyledPanel)
        self.messagesFrame.setFrameShadow(QtGui.QFrame.Sunken)
        self.messagesFrame.setObjectName(_fromUtf8("messagesFrame"))
        self.verticalLayout = QtGui.QVBoxLayout(self.messagesFrame)
        self.verticalLayout.setMargin(0)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.messagesWebView = QtWebKit.QWebView(self.messagesFrame)
        self.messagesWebView.setUrl(QtCore.QUrl(_fromUtf8("about:blank")))
        self.messagesWebView.setObjectName(_fromUtf8("messagesWebView"))
        self.verticalLayout.addWidget(self.messagesWebView)
        self.verticalLayout_2.addWidget(self.messagesFrame)
        self.horizontalLayout = QtGui.QHBoxLayout()
        self.horizontalLayout.setObjectName(_fromUtf8("horizontalLayout"))
        self.messageTextEdit = QtGui.QPlainTextEdit(QHangupsConversationWidget)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.messageTextEdit.sizePolicy().hasHeightForWidth())
        self.messageTextEdit.setSizePolicy(sizePolicy)
        self.messageTextEdit.setMaximumSize(QtCore.QSize(16777215, 60))
        self.messageTextEdit.setObjectName(_fromUtf8("messageTextEdit"))
        self.horizontalLayout.addWidget(self.messageTextEdit)
        self.sendButton = QtGui.QPushButton(QHangupsConversationWidget)
        self.sendButton.setMaximumSize(QtCore.QSize(16777215, 60))
        self.sendButton.setObjectName(_fromUtf8("sendButton"))
        self.horizontalLayout.addWidget(self.sendButton)
        self.verticalLayout_2.addLayout(self.horizontalLayout)

        self.retranslateUi(QHangupsConversationWidget)
        QtCore.QMetaObject.connectSlotsByName(QHangupsConversationWidget)
        QHangupsConversationWidget.setTabOrder(self.messageTextEdit, self.sendButton)
        QHangupsConversationWidget.setTabOrder(self.sendButton, self.messagesWebView)

    def retranslateUi(self, QHangupsConversationWidget):
        QHangupsConversationWidget.setWindowTitle(_translate("QHangupsConversationWidget", "QHangups - Conversation", None))
        self.sendButton.setText(_translate("QHangupsConversationWidget", "Send", None))
        self.sendButton.setShortcut(_translate("QHangupsConversationWidget", "Ctrl+Return", None))

from PyQt4 import QtWebKit



import datetime, asyncio

from PyQt4 import QtCore, QtGui, QtWebKit

import hangups
from hangups.ui.utils import get_conv_name

from qhangups.utils import text_to_segments, message_to_html
from qhangups.ui_qhangupsconversationwidget import Ui_QHangupsConversationWidget


class QHangupsConversationWidget(QtGui.QWidget, Ui_QHangupsConversationWidget):
    """Conversation tab"""
    def __init__(self, tab_parent, client, conv, parent=None):
        super().__init__(parent)
        self.setupUi(self)

        self.tab_parent = tab_parent
        self.client = client
        self.conv = conv
        self.messages_id_list = []

        self.client.on_disconnect.add_observer(self.on_disconnect)
        self.client.on_reconnect.add_observer(self.on_reconnect)
        self.conv.on_event.add_observer(self.on_event)
        self.conv.on_watermark_notification.add_observer(self.on_watermark_notification)

        self.messageTextEdit.textChanged.connect(self.on_text_changed)
        self.sendButton.clicked.connect(self.on_send_clicked)
        self.messagesWebView.page().mainFrame().contentsSizeChanged.connect(self.on_contents_size_changed)
        self.messagesWebView.page().linkClicked.connect(self.on_link_clicked)

        settings = QtCore.QSettings()
        self.enter_send_message = settings.value("enter_send_message", False, type=bool)

        # Install ourselves as event filter so we can catch Enter key press (see eventFilter method)
        self.messageTextEdit.installEventFilter(self)

        # Initialize QWebView with list of messages
        self.init_messages()

        self.num_unread_local = 0
        for event in self.conv.events:
            self.on_event(event, set_title=False, set_unread=False)

    def eventFilter(self, obj, event):
        """Event filter for catching Enter key press and sending message"""
        if self.enter_send_message and obj is self.messageTextEdit:
            if event.type() == QtCore.QEvent.KeyPress and event.key() == QtCore.Qt.Key_Return:
                self.on_send_clicked()
                return True
        return super().eventFilter(obj, event)

    def get_num_unread(self, local_unread=False):
        """Get number of unread messages (server-side or local)"""
        settings = QtCore.QSettings()
        if not settings.value("send_read_state", True, type=bool) or local_unread:
            num_unread = self.num_unread_local
        else:
            num_unread = len([conv_event for conv_event in self.conv.unread_events if
                              isinstance(conv_event, hangups.ChatMessageEvent) and
                              not self.conv.get_user(conv_event.user_id).is_self])
        return num_unread

    def set_title(self):
        """Update this conversation's tab title."""
        title = get_conv_name(self.conv, truncate=True)
        conv_widget_id = self.tab_parent.conversationsTabWidget.indexOf(self)
        num_unread = self.get_num_unread()
        if num_unread > 0:
            title += ' ({})'.format(num_unread)
            self.tab_parent.conversationsTabWidget.tabBar().setTabTextColor(conv_widget_id, QtCore.Qt.darkBlue)
        else:
            self.tab_parent.conversationsTabWidget.tabBar().setTabTextColor(conv_widget_id, QtGui.QColor())
        self.tab_parent.conversationsTabWidget.setTabText(conv_widget_id, title)
        self.tab_parent.conversationsTabWidget.setTabToolTip(conv_widget_id, title)

    def init_messages(self):
        """Initialize QWebView with list of messages"""
        self.messagesWebView.setContextMenuPolicy(QtCore.Qt.NoContextMenu)
        self.messagesWebView.page().setLinkDelegationPolicy(QtWebKit.QWebPage.DelegateAllLinks)
        #self.messagesWebView.settings().setAttribute(QtWebKit.QWebSettings.LocalContentCanAccessRemoteUrls, True)
        self.messagesWebView.setHtml(
            """<!doctype html>
            <html lang="en">
            <head>
                <meta charset="utf-8">
                <title>Messages</title>
                <style>
                    html { font-family: sans-serif; font-size: 10pt; }
                    div.message { margin-bottom: 1em; }
                </style>
            </head>
            <body>
                <div id="messages"></div>
            </body>
            </html>"""
        )

    def add_message(self, timestamp, text, username=None, user_id=None, message_id=None):
        """Add new message to list of messages"""
        # Check for already existing messages (so we avoid showing duplicates)
        if message_id is not None and message_id in self.messages_id_list:
            return
        else:
            self.messages_id_list.append(message_id)

        datestr = "%d.%m. %H:%M" if timestamp.astimezone(tz=None).date() < datetime.date.today() else "%H:%M"
        link = "https://plus.google.com/u/0/{}/about".format(user_id) if user_id else ""
        message = ("<b>{}{}:</b><br>\n{}<br>\n").format(
            timestamp.astimezone(tz=None).strftime(datestr),
            """ | <a href="{}">{}</a>""".format(link, username) if username is not None else "",
            text
        )
        doc = self.messagesWebView.page().mainFrame().documentElement()
        doc.findFirst("div[id=messages]").appendInside(
            """<div class="message">{}</div>""".format(message)
        )

    def is_current(self):
        """Is this conversation in current tab?"""
        return self.tab_parent.conversationsTabWidget.currentWidget() is self

    def set_active(self):
        """Activate conversation tab"""
        settings = QtCore.QSettings()

        # Set the client as active
        if settings.value("send_client_active", True, type=bool):
            future = asyncio.async(self.client.set_active())
            future.add_done_callback(lambda future: future.result())

        # Mark the newest event as read
        if settings.value("send_read_state", True, type=bool):
            future = asyncio.async(self.conv.update_read_timestamp())
            future.add_done_callback(lambda future: future.result())

        self.num_unread_local = 0
        self.set_title()
        self.messageTextEdit.setFocus()

    def on_contents_size_changed(self, size):
        """Size of contents in messagesWebView changed (callback)"""
        page = self.messagesWebView.page()
        viewport_height = page.viewportSize().height()
        scroll_position = page.mainFrame().scrollPosition().y()

        # Compute max. scroll position manually, because
        # scrollBarMaximum(QtCore.Qt.Vertical) doesn't work here
        scroll_max = page.mainFrame().contentsSize().height() - viewport_height

        # If user has scrolled more than half of viewport away, don't scroll down automatically
        if scroll_position < (scroll_max - 0.5 * viewport_height):
            return

        # Use singl-shot timer, because scrolling doesn't work here (maybe some Qt bug?)
        QtCore.QTimer.singleShot(0, self.on_contents_size_changed_timeout)

    def on_contents_size_changed_timeout(self):
        """Scroll to the bottom after timeout of single-shot timer (callback)"""
        frame = self.messagesWebView.page().mainFrame()
        frame.setScrollPosition(QtCore.QPoint(0, frame.scrollBarMaximum(QtCore.Qt.Vertical)))

    def on_link_clicked(self, url):
        """Open links in external web browser (callback)"""
        QtGui.QDesktopServices.openUrl(url)

    def on_text_changed(self):
        """Message text changed (callback)"""
        pass

    def on_send_clicked(self):
        """Send button pressed (callback)"""
        text = self.messageTextEdit.toPlainText()
        if not text.strip():
            return

        self.messageTextEdit.setEnabled(False)
        self.sendButton.setEnabled(False)

        segments = text_to_segments(text)
        asyncio.async(
            self.conv.send_message(segments)
        ).add_done_callback(self.on_message_sent)

    def on_message_sent(self, future):
        """Handle showing an error if a message fails to send (callback)"""
        try:
            future.result()
        except hangups.NetworkError:
            QtGui.QMessageBox.warning(self, self.tr("QHangups - Warning"),
                                      self.tr("Failed to send message!"))
        else:
            self.messageTextEdit.clear()
        finally:
            self.messageTextEdit.setEnabled(True)
            self.sendButton.setEnabled(True)
            self.messageTextEdit.setFocus()

    def on_disconnect(self):
        """Show that Hangups has disconnected from server (callback)"""
        self.add_message(datetime.datetime.now(tz=datetime.timezone.utc), "<i>*** disconnected ***</i>")

    def on_reconnect(self):
        """Show that Hangups has reconnected to server (callback)"""
        self.add_message(datetime.datetime.now(tz=datetime.timezone.utc), "<i>*** connected ***</i>")

    def on_watermark_notification(self, watermark_notification):
        """Update unread count after receiving watermark notification (callback)"""
        self.set_title()

    def on_event(self, conv_event, set_title=True, set_unread=True):
        """Hangups event received (callback)"""
        user = self.conv.get_user(conv_event.user_id)

        if isinstance(conv_event, hangups.ChatMessageEvent):
            self.handle_message(conv_event, user, set_unread=set_unread)
        elif isinstance(conv_event, hangups.RenameEvent):
            self.handle_rename(conv_event, user)
        elif isinstance(conv_event, hangups.MembershipChangeEvent):
            self.handle_membership_change(conv_event, user)

        # Update the title in case unread count or conversation name changed.
        if set_title:
            self.set_title()

    def handle_message(self, conv_event, user, set_unread=True):
        """Handle received chat message"""
        self.add_message(conv_event.timestamp, message_to_html(conv_event),
                         user.full_name, user.id_.chat_id, conv_event.id_)
        # Update the count of unread messages.
        if not user.is_self and set_unread and not self.is_current():
            self.num_unread_local += 1

    def handle_rename(self, conv_event, user):
        """Handle received rename event"""
        if conv_event.new_name == '':
            text = '<i>*** cleared the conversation name ***</i>'
        else:
            text = '<i>*** renamed the conversation to {} ***</i>'.format(conv_event.new_name)
        self.add_message(conv_event.timestamp, text, user.full_name, user.id_.chat_id, conv_event.id_)

    def handle_membership_change(self, conv_event, user):
        """Handle received membership change event"""
        event_users = [self.conv.get_user(user_id) for user_id in conv_event.participant_ids]
        names = ', '.join(user.full_name for user in event_users)
        if conv_event.type_ == hangups.MembershipChangeType.JOIN:
            self.add_message(conv_event.timestamp,
                             '<i>*** added {} to the conversation ***</i>'.format(names),
                             user.full_name, user.id_.chat_id, conv_event.id_)
        else:
            for name in names:
                self.add_message(conv_event.timestamp,
                                 '<i>*** left the conversation ***</i>',
                                 user.full_name, user.id_.chat_id, conv_event.id_)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'qhangups/qhangupsconversations.ui'
#
# Created: Wed May 20 12:38:46 2015
#      by: PyQt4 UI code generator 4.11.3
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)

class Ui_QHangupsConversations(object):
    def setupUi(self, QHangupsConversations):
        QHangupsConversations.setObjectName(_fromUtf8("QHangupsConversations"))
        QHangupsConversations.resize(500, 350)
        self.centralwidget = QtGui.QWidget(QHangupsConversations)
        self.centralwidget.setObjectName(_fromUtf8("centralwidget"))
        self.gridLayout = QtGui.QGridLayout(self.centralwidget)
        self.gridLayout.setObjectName(_fromUtf8("gridLayout"))
        self.conversationsTabWidget = QtGui.QTabWidget(self.centralwidget)
        self.conversationsTabWidget.setElideMode(QtCore.Qt.ElideRight)
        self.conversationsTabWidget.setTabsClosable(True)
        self.conversationsTabWidget.setMovable(True)
        self.conversationsTabWidget.setObjectName(_fromUtf8("conversationsTabWidget"))
        self.gridLayout.addWidget(self.conversationsTabWidget, 0, 0, 1, 1)
        QHangupsConversations.setCentralWidget(self.centralwidget)
        self.menubar = QtGui.QMenuBar(QHangupsConversations)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 500, 27))
        self.menubar.setObjectName(_fromUtf8("menubar"))
        QHangupsConversations.setMenuBar(self.menubar)
        self.statusbar = QtGui.QStatusBar(QHangupsConversations)
        self.statusbar.setObjectName(_fromUtf8("statusbar"))
        QHangupsConversations.setStatusBar(self.statusbar)

        self.retranslateUi(QHangupsConversations)
        self.conversationsTabWidget.setCurrentIndex(-1)
        QtCore.QMetaObject.connectSlotsByName(QHangupsConversations)

    def retranslateUi(self, QHangupsConversations):
        QHangupsConversations.setWindowTitle(_translate("QHangupsConversations", "QHangups - Conversations", None))




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'qhangups/qhangupsconversationslist.ui'
#
# Created: Wed May 20 12:38:45 2015
#      by: PyQt4 UI code generator 4.11.3
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)

class Ui_QHangupsConversationsList(object):
    def setupUi(self, QHangupsConversationsList):
        QHangupsConversationsList.setObjectName(_fromUtf8("QHangupsConversationsList"))
        QHangupsConversationsList.resize(250, 500)
        self.centralwidget = QtGui.QWidget(QHangupsConversationsList)
        self.centralwidget.setObjectName(_fromUtf8("centralwidget"))
        self.verticalLayout = QtGui.QVBoxLayout(self.centralwidget)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.conversationsListWidget = QtGui.QListWidget(self.centralwidget)
        self.conversationsListWidget.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.conversationsListWidget.setObjectName(_fromUtf8("conversationsListWidget"))
        self.verticalLayout.addWidget(self.conversationsListWidget)
        QHangupsConversationsList.setCentralWidget(self.centralwidget)
        self.menubar = QtGui.QMenuBar(QHangupsConversationsList)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 250, 27))
        self.menubar.setObjectName(_fromUtf8("menubar"))
        QHangupsConversationsList.setMenuBar(self.menubar)
        self.statusbar = QtGui.QStatusBar(QHangupsConversationsList)
        self.statusbar.setObjectName(_fromUtf8("statusbar"))
        QHangupsConversationsList.setStatusBar(self.statusbar)

        self.retranslateUi(QHangupsConversationsList)
        QtCore.QMetaObject.connectSlotsByName(QHangupsConversationsList)

    def retranslateUi(self, QHangupsConversationsList):
        QHangupsConversationsList.setWindowTitle(_translate("QHangupsConversationsList", "QHangups", None))




import os, glob

from PyQt4 import QtCore, QtGui

from qhangups.ui_qhangupssettings import Ui_QHangupsSettings


class QHangupsSettings(QtGui.QDialog, Ui_QHangupsSettings):
    """Settings dialog"""
    def __init__(self, parent=None):
        super().__init__(parent)
        self.setupUi(self)
        self.populate_ui()

    def populate_ui(self):
        """Load settings and populate dialog"""
        settings = QtCore.QSettings()

        self.sendClientActiveCheckBox.setChecked(settings.value("send_client_active", True, type=bool))
        self.sendReadStateCheckBox.setChecked(settings.value("send_read_state", True, type=bool))
        self.enterSendMessageCheckBox.setChecked(settings.value("enter_send_message", False, type=bool))

        lang_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), "languages")

        self.languageComboBox.clear()
        self.languageComboBox.addItem(QtCore.QLocale.languageToString(QtCore.QLocale("en").language()), "en")

        for f in glob.glob(os.path.join(lang_path, "*.qm")):
            file_locale = os.path.splitext(os.path.basename(f))[0].split("_")[-1]
            file_lang = QtCore.QLocale.languageToString(QtCore.QLocale(file_locale).language())
            self.languageComboBox.addItem(file_lang, file_locale)

        language = settings.value("language")
        if not language:
            language = QtCore.QLocale.system().name().split("_")[0]

        i = self.languageComboBox.findData(language)
        if i == -1:
            self.languageComboBox.setCurrentIndex(0)
        else:
            self.languageComboBox.setCurrentIndex(i)

    def accept(self):
        """Save settings after clicking on OK button"""
        settings = QtCore.QSettings()
        settings.setValue("send_client_active", self.sendClientActiveCheckBox.isChecked())
        settings.setValue("send_read_state", self.sendReadStateCheckBox.isChecked())
        settings.setValue("enter_send_message", self.enterSendMessageCheckBox.isChecked())
        settings.setValue("language", self.languageComboBox.itemData(self.languageComboBox.currentIndex()))
        QtGui.QDialog.accept(self)



#!/usr/bin/env python

import sys, os, logging, argparse, asyncio, signal

from PyQt4 import QtCore, QtGui
import appdirs
import hangups
from hangups.ui.notify import Notifier

# Force use of PyQt4 for Quamash (until we are PyQt5 compatible)
os.environ['QUAMASH_QTIMPL'] = "PyQt4"

# Fake os.name to be always "posix" when loading QEventLoop.
# We need SelectorEventLoop on Windows because
# ProactorEventLoop doesn't support SSL in Python < 3.5
# (and Quamash uses ProactorEventLoop on Windows by default)
_os_name_orig = os.name
os.name = "posix"
from quamash import QEventLoop
os.name = _os_name_orig

from qhangups.version import __version__
from qhangups.settings import QHangupsSettings
from qhangups.conversations import QHangupsConversations
from qhangups.conversationslist import QHangupsConversationsList
from qhangups.browser import QHangupsBrowser


# Basic settings
LOG_FORMAT = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'

# Prepare Qt translators
translator = QtCore.QTranslator()
qt_translator = QtCore.QTranslator()


class QHangupsMainWidget(QtGui.QWidget):
    """QHangups main widget (icon in system tray)"""
    startHangups = QtCore.pyqtSignal()
    stopHangups = QtCore.pyqtSignal()

    def __init__(self, refresh_token_path, parent=None):
        super().__init__(parent)
        self.set_language()

        self.refresh_token_path = refresh_token_path
        self.hangups_running = False
        self.client = None

        self.create_actions()
        self.create_menu()
        self.create_icon()
        self.update_status()

        # These are populated by on_connect when it's called.
        self.conv_list = None  # hangups.ConversationList
        self.user_list = None  # hangups.UserList
        self.notifier = None   # hangups.notify.Notifier

        # Widgets
        self.conversations_dialog = QHangupsConversationsList(controller=self)
        self.messages_dialog = QHangupsConversations(controller=self)

        # Setup system tray icon doubleclick timer
        self.icon_doubleclick_timer = QtCore.QTimer(self)
        self.icon_doubleclick_timer.setSingleShot(True)
        self.icon_doubleclick_timer.timeout.connect(self.icon_doubleclick_timeout)

        # Handle signals on Unix
        # (add_signal_handler is not implemented on Windows)
        try:
            loop = asyncio.get_event_loop()
            for signum in (signal.SIGINT, signal.SIGTERM):
                loop.add_signal_handler(signum, lambda: self.quit(force=True))
        except NotImplementedError:
            pass

    def create_actions(self):
        """Create actions and connect relevant signals"""
        self.startAction = QtGui.QAction(self)
        self.startAction.triggered.connect(self.hangups_start)
        self.stopAction = QtGui.QAction(self)
        self.stopAction.triggered.connect(self.hangups_stop)
        self.settingsAction = QtGui.QAction(self)
        self.settingsAction.triggered.connect(self.settings)
        self.aboutAction = QtGui.QAction(self)
        self.aboutAction.triggered.connect(self.about)
        self.quitAction = QtGui.QAction(self)
        self.quitAction.triggered.connect(self.quit)

    def create_menu(self):
        """Create menu and add items to it"""
        self.trayIconMenu = QtGui.QMenu(self)
        self.trayIconMenu.addAction(self.startAction)
        self.trayIconMenu.addAction(self.stopAction)
        self.trayIconMenu.addSeparator()
        self.trayIconMenu.addAction(self.settingsAction)
        self.trayIconMenu.addAction(self.aboutAction)
        self.trayIconMenu.addSeparator()
        self.trayIconMenu.addAction(self.quitAction)

    def create_icon(self):
        """Create system tray icon"""
        self.trayIcon = QtGui.QSystemTrayIcon(self)
        self.iconActive = QtGui.QIcon("{}/qhangups.svg".format(os.path.dirname(os.path.abspath(__file__))))
        self.iconDisabled = QtGui.QIcon("{}/qhangups_disabled.svg".format(os.path.dirname(os.path.abspath(__file__))))
        self.trayIcon.activated.connect(self.icon_activated)
        self.trayIcon.setContextMenu(self.trayIconMenu)
        self.trayIcon.setIcon(self.iconDisabled)
        self.trayIcon.setToolTip("QHangups")
        self.trayIcon.show()

    def retranslateUi(self):
        """Retranslate GUI"""
        self.startAction.setText(self.tr("&Connect"))
        self.stopAction.setText(self.tr("&Disconnect"))
        self.settingsAction.setText(self.tr("S&ettings ..."))
        self.aboutAction.setText(self.tr("A&bout ..."))
        self.quitAction.setText(self.tr("&Quit"))

    def login(self, refresh_token_path):
        """Login to Google account"""
        try:
            cookies = hangups.auth.get_auth(self.get_credentials, refresh_token_path)
            return cookies
        except hangups.GoogleAuthError:
            QtGui.QMessageBox.warning(self, self.tr("QHangups - Warning"),
                                      self.tr("Google login failed!"))
            return False

    def get_credentials(self):
        """Ask user for OAuth 2 authorization code"""
        browser = QHangupsBrowser(hangups.auth.OAUTH2_LOGIN_URL, self).exec_()
        code, ok = QtGui.QInputDialog.getText(self, self.tr("QHangups - Authorization"),
                                              self.tr("Authorization code:"),
                                              QtGui.QLineEdit.Normal)
        if ok and code:
            return code
        else:
            return None

    def update_status(self):
        """Update GUI according to Hangups status"""
        if self.hangups_running:
            self.trayIcon.setIcon(self.iconActive)
            self.startAction.setEnabled(False)
            self.stopAction.setEnabled(True)
        else:
            self.trayIcon.setIcon(self.iconDisabled)
            self.startAction.setEnabled(True)
            self.stopAction.setEnabled(False)

    def hangups_start(self):
        """Connect to Hangouts"""
        cookies = self.login(self.refresh_token_path)
        if cookies:
            self.startHangups.emit()

            self.client = hangups.Client(cookies)
            self.client.on_connect.add_observer(self.on_connect)

            # Run Hangups event loop
            asyncio.async(
                self.client.connect()
            ).add_done_callback(lambda future: future.result())
            self.hangups_running = True
            self.update_status()

    def hangups_stop(self):
        """Disconnect from Hangouts"""
        self.stopHangups.emit()

        asyncio.async(
            self.client.disconnect()
        ).add_done_callback(lambda future: future.result())

        self.conv_list = None
        self.user_list = None
        self.notifier = None

        self.hangups_running = False
        self.client = None
        self.update_status()

    def about(self):
        """Show About dialog"""
        QtGui.QMessageBox.information(self, self.tr("About"), self.tr("QHangups {}".format(__version__)))

    def settings(self):
        """Show Settings dialog"""
        dialog = QHangupsSettings(self)
        if dialog.exec_():
            self.set_language()
            if self.hangups_running:
                self.hangups_stop()
                self.hangups_start()

    def set_language(self):
        """Change language"""
        settings = QtCore.QSettings()

        language = settings.value("language")
        if not language:
            language = QtCore.QLocale.system().name().split("_")[0]

        lang_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), "languages")
        lang_file = "qhangups_{}.qm".format(language)

        qt_lang_path = QtCore.QLibraryInfo.location(QtCore.QLibraryInfo.TranslationsPath)
        qt_lang_file = "qt_{}.qm".format(language)

        if os.path.isfile(os.path.join(lang_path, lang_file)):
            translator.load(lang_file, lang_path)
            qt_translator.load(qt_lang_file, qt_lang_path)
        else:
            translator.load("")
            qt_translator.load("")

    def icon_activated(self, reason):
        """Connect or disconnect from Hangouts by double-click on tray icon"""
        if reason == QtGui.QSystemTrayIcon.Trigger or reason == QtGui.QSystemTrayIcon.DoubleClick:
            if self.icon_doubleclick_timer.isActive():
                self.icon_doubleclick_timer.stop()
                if self.hangups_running:
                    self.hangups_stop()
                else:
                    self.hangups_start()
            else:
                self.icon_doubleclick_timer.start(QtGui.qApp.doubleClickInterval())

    def icon_doubleclick_timeout(self):
        """Open or close list of conversations after single-click on tray icon"""
        if self.conversations_dialog:
            if self.conversations_dialog.isVisible() and not self.conversations_dialog.isMinimized():
                self.conversations_dialog.hide()
            else:
                self.conversations_dialog.showNormal()
                self.conversations_dialog.raise_()
                self.conversations_dialog.activateWindow()

    def quit(self, force=False):
        """Quit QHangups"""
        if self.hangups_running:
            if not force:
                reply = QtGui.QMessageBox.question(self, self.tr("QHangups - Quit"),
                                                   self.tr("You are still connected to Google Hangouts. "
                                                           "Do you really want to quit QHangups?"),
                                                   QtGui.QMessageBox.Yes | QtGui.QMessageBox.No, QtGui.QMessageBox.No)
                if reply != QtGui.QMessageBox.Yes:
                    return
            self.hangups_stop()

        loop = asyncio.get_event_loop()
        loop.stop()
        # QtGui.qApp.quit()

    def changeEvent(self, event):
        """Handle LanguageChange event"""
        if (event.type() == QtCore.QEvent.LanguageChange):
            print("Language changed")
            self.retranslateUi()

        super().changeEvent(event)

    def open_messages_dialog(self, conv_id, switch=True):
        """Open conversation in new tab"""
        self.messages_dialog.set_conv_tab(conv_id, switch=switch)
        self.messages_dialog.showNormal()
        if switch:
            self.messages_dialog.raise_()
            self.messages_dialog.activateWindow()

    @asyncio.coroutine
    def on_connect(self, initial_data):
        """Handle connecting for the first time (callback)"""
        print('Connected')
        self.user_list = yield from hangups.build_user_list(
            self.client,
            initial_data
        )
        self.conv_list = hangups.ConversationList(
            self.client,
            initial_data.conversation_states,
            self.user_list,
            initial_data.sync_timestamp
        )
        self.conv_list.on_event.add_observer(self.on_event)

        # Setup notifications
        self.notifier = Notifier(self.conv_list)

        # Setup conversations window
        self.messages_dialog.init_conversations(self.client, self.conv_list)

        # Setup conversations list window and show it
        self.conversations_dialog.init_conversations(self.client, self.conv_list)
        self.conversations_dialog.show()

    @asyncio.coroutine
    def on_event(self, conv_event):
        """Open conversation tab for new messages when they arrive (callback)"""
        if isinstance(conv_event, hangups.ChatMessageEvent):
            self.open_messages_dialog(conv_event.conversation_id, switch=False)


def main():
    # Build default paths for files.
    dirs = appdirs.AppDirs('QHangups', 'QHangups')
    default_log_path = os.path.join(dirs.user_data_dir, 'hangups.log')
    default_token_path = os.path.join(dirs.user_data_dir, 'refresh_token.txt')

    # Setup command line argument parser
    parser = argparse.ArgumentParser(prog='qhangups',
                                     formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('-d', '--debug', action='store_true',
                        help='log detailed debugging messages')
    parser.add_argument('--log', default=default_log_path,
                        help='log file path')
    parser.add_argument('--token', default=default_token_path,
                        help='OAuth refresh token storage path')
    args = parser.parse_args()

    # Create all necessary directories.
    for path in [args.log, args.token]:
        directory = os.path.dirname(path)
        if directory and not os.path.isdir(directory):
            try:
                os.makedirs(directory)
            except OSError as e:
                sys.exit('Failed to create directory: {}'.format(e))

    # Setup logging
    log_level = logging.DEBUG if args.debug else logging.WARNING
    logging.basicConfig(filename=args.log, level=log_level, format=LOG_FORMAT)
    # asyncio's debugging logs are VERY noisy, so adjust the log level
    logging.getLogger('asyncio').setLevel(logging.WARNING)
    # ...and if we don't need Hangups debug logs, then uncomment this:
    # logging.getLogger('hangups').setLevel(logging.WARNING)

    # Setup QApplication
    app = QtGui.QApplication(sys.argv)
    app.setOrganizationName("QHangups")
    app.setOrganizationDomain("qhangups.eutopia.cz")
    app.setApplicationName("QHangups")
    app.setQuitOnLastWindowClosed(False)
    app.installTranslator(translator)
    app.installTranslator(qt_translator)

    # Start Quamash event loop
    loop = QEventLoop(app)
    asyncio.set_event_loop(loop)
    with loop:
        widget = QHangupsMainWidget(args.token)
        loop.run_forever()


if __name__ == "__main__":
    main()



from PyQt4 import QtCore, QtGui

from qhangups.conversationwidget import QHangupsConversationWidget
from qhangups.ui_qhangupsconversations import Ui_QHangupsConversations


class QHangupsConversations(QtGui.QMainWindow, Ui_QHangupsConversations):
    """Tabbed window with opened conversations"""
    def __init__(self, controller, parent=None):
        super().__init__(parent)
        self.setupUi(self)

        self.controller = controller
        self.client = None
        self.conv_list = None
        self.conv_widgets = {}

        self.controller.stopHangups.connect(self.on_stop)
        self.conversationsTabWidget.currentChanged.connect(self.on_tab_current_changed)
        self.conversationsTabWidget.tabCloseRequested.connect(self.on_tab_close_requested)

        # Install ourselves as event filter so we can catch middle mouse button (see eventFilter method)
        self.conversationsTabWidget.tabBar().installEventFilter(self)

    def init_conversations(self, client, conv_list):
        """Initialize list of conversations"""
        self.client = client
        self.conv_list = conv_list

    def eventFilter(self, obj, event):
        """Event filter for catching middle mouse button on tab and closing it"""
        if obj is self.conversationsTabWidget.tabBar():
            if event.type() == QtCore.QEvent.MouseButtonRelease and event.button() == QtCore.Qt.MidButton:
                tab_index = obj.tabAt(event.pos())
                if tab_index != -1:
                    obj.tabCloseRequested.emit(tab_index)
                    return True
        return super().eventFilter(obj, event)

    def get_conv_widget(self, conv_id):
        """Return an existing or new QHangupsConversationWidget"""
        if conv_id not in self.conv_widgets:
            conv = self.conv_list.get(conv_id)
            conv_widget = QHangupsConversationWidget(self, self.client, conv)
            self.conv_widgets[conv_id] = conv_widget
            self.conversationsTabWidget.addTab(conv_widget, "")
            conv_widget.set_title()
        return self.conv_widgets[conv_id]

    def set_conv_tab(self, conv_id, switch=False):
        """Add conversation tab (if not present) and optionally switch to it"""
        conv_widget = self.get_conv_widget(conv_id)
        conv_widget_id = self.conversationsTabWidget.indexOf(conv_widget)

        if switch:
            self.conversationsTabWidget.setCurrentWidget(conv_widget)

    def on_stop(self):
        """Remove all QHangupsConversationWidgets after stopping Hangups (callback)"""
        self.conversationsTabWidget.clear()
        self.conv_widgets = {}
        self.client = None
        self.conv_list = None
        self.close()

    def on_tab_current_changed(self, conv_widget_id):
        """Current tab changed (callback)"""
        conv_widget = self.conversationsTabWidget.widget(conv_widget_id)
        if conv_widget:
            conv_widget.set_active()

    def on_tab_close_requested(self, conv_widget_id):
        """Tab close button clicked (callback)"""
        conv_widget = self.conversationsTabWidget.widget(conv_widget_id)
        self.conversationsTabWidget.removeTab(conv_widget_id)
        del self.conv_widgets[conv_widget.conv.id_]
        if not self.conv_widgets:
            self.close()

    def save_geometry(self):
        """Save window position and size"""
        settings = QtCore.QSettings()
        settings.setValue("conversations_geometry", self.saveGeometry())

    def restore_geometry(self):
        """Restore window position and size from saved values"""
        settings = QtCore.QSettings()
        if settings.value("conversations_geometry"):
            self.restoreGeometry(settings.value("conversations_geometry"))

    def showEvent(self, event):
        """Restore window geometry when window is displayed"""
        # If we don't use single-shot timer, position is not restored
        # correctly from minimized state on Windows
        QtCore.QTimer.singleShot(0, self.restore_geometry)
        super().showEvent(event)

    def hideEvent(self, event):
        """Save window geometry when window is hidden"""
        self.save_geometry()
        super().hideEvent(event)



from PyQt4 import QtCore, QtGui

import hangups
from hangups.ui.utils import get_conv_name

from qhangups.ui_qhangupsconversationslist import Ui_QHangupsConversationsList


class QHangupsConversationsList(QtGui.QMainWindow, Ui_QHangupsConversationsList):
    """Window with list of conversations"""
    def __init__(self, controller, parent=None):
        super().__init__(parent)
        self.setupUi(self)

        self.controller = controller
        self.client = None
        self.conv_list = None

        self.controller.startHangups.connect(self.on_start)
        self.controller.stopHangups.connect(self.on_stop)

        self.set_status(self.tr("Disconnected"))

    def init_conversations(self, client, conv_list):
        """Initialize list of conversations and connect signals"""
        self.client = client
        self.conv_list = conv_list

        self.conv_list.on_event.add_observer(self.on_event)
        self.client.on_disconnect.add_observer(self.on_disconnect)
        self.client.on_reconnect.add_observer(self.on_reconnect)
        self.conversationsListWidget.itemActivated.connect(self.on_item_activated)

        self.update_conversations()

    def set_status(self, status_text):
        """Display static status text instead of list of conversations"""
        self.conversationsListWidget.clear()
        item = QtGui.QListWidgetItem(status_text)
        item.setTextAlignment(QtCore.Qt.AlignHCenter)
        self.conversationsListWidget.addItem(item)

    def update_conversations(self):
        """Update list of conversations"""
        self.conversationsListWidget.clear()
        for conv in sorted(self.conv_list.get_all(), reverse=True, key=lambda c: c.last_modified):
            item = QtGui.QListWidgetItem(get_conv_name(conv, truncate=True))
            item.setToolTip(get_conv_name(conv))
            item.setData(QtCore.Qt.UserRole, conv.id_)
            self.conversationsListWidget.addItem(item)

    def on_item_activated(self, item):
        """List item activated (callback)"""
        item_data = item.data(QtCore.Qt.UserRole)
        if item_data:
            self.controller.open_messages_dialog(item_data)

    def on_event(self, conv_event):
        """Hangups event received (callback)"""
        if isinstance(conv_event, hangups.RenameEvent):
            self.update_conversations()

    def on_disconnect(self):
        """Show that Hangups has disconnected from server (callback)"""
        self.set_status(self.tr("Reconnecting..."))

    def on_reconnect(self):
        """Show that Hangups has reconnected to server (callback)"""
        self.update_conversations()

    def on_start(self):
        """Show that Hangups is starting (callback)"""
        self.set_status(self.tr("Connecting..."))

    def on_stop(self):
        """Show that Hangups has been stopped (callback)"""
        self.client = None
        self.conv_list = None
        self.set_status(self.tr("Disconnected"))

    def save_geometry(self):
        """Save window position and size"""
        settings = QtCore.QSettings()
        settings.setValue("conversationslist_geometry", self.saveGeometry())

    def restore_geometry(self):
        """Restore window position and size from saved values"""
        settings = QtCore.QSettings()
        if settings.value("conversationslist_geometry"):
            self.restoreGeometry(settings.value("conversationslist_geometry"))

    def showEvent(self, event):
        """Restore window geometry when window is displayed"""
        # If we don't use single-shot timer, position is not restored
        # correctly from minimized state on Windows
        QtCore.QTimer.singleShot(0, self.restore_geometry)
        super().showEvent(event)

    def hideEvent(self, event):
        """Save window geometry when window is hidden"""
        self.save_geometry()
        super().hideEvent(event)



from PyQt4 import QtCore, QtGui, QtWebKit

from qhangups.ui_qhangupsbrowser import Ui_QHangupsBrowser

class QHangupsBrowser(QtGui.QDialog, Ui_QHangupsBrowser):
    """Extremely simple web browser"""
    def __init__(self, url="", parent=None):
        super().__init__(parent)
        self.setupUi(self)

        if url:
            self.load(url)

    def load(self, url):
        """Load URL"""
        self.browserWebView.load(QtCore.QUrl.fromUserInput(url))



#!/usr/bin/env python

from setuptools import setup
from qhangups.version import __version__

setup(
    name="QHangups",
    version=__version__,
    description="Alternative client for Google Hangouts written in PyQt",
    author="Michal Krenek (Mikos)",
    author_email="m.krenek@gmail.com",
    url="https://github.com/xmikos/qhangups",
    license="GNU GPLv3",
    packages=["qhangups"],
    package_data={
        "qhangups": [
            "qhangups.svg",
            "qhangups_disabled.svg",
            "*.ui",
            "languages/*.qm",
            "languages/*.ts"
        ]
    },
    data_files=[
        ("share/applications", ["qhangups.desktop"]),
        ("share/pixmaps", ["qhangups.png"])
    ],
    entry_points={
        "gui_scripts": [
            "qhangups=qhangups.__main__:main"
        ],
    },
    install_requires=[
        "hangups>=0.2.8",
        "appdirs",
        "Quamash",
        "asyncio"  # Only needed for backward compatibility with Python 3.3
    ],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: MacOS X",
        "Environment :: Win32 (MS Windows)",
        "Environment :: X11 Applications :: Qt",
        "Intended Audience :: End Users/Desktop",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Topic :: Communications :: Chat"
    ]
)



import logging, html

import hangups

logger = logging.getLogger(__name__)


def text_to_segments(text):
    """Create list of ChatMessageSegments from text"""
    return hangups.ChatMessageSegment.from_str(text)


def segment_to_html(segment):
    """Create simple HTML from ChatMessageSegment"""
    text = html.escape(segment.text) if segment.text else ""
    text = text.replace('\n', '<br>\n')

    message = []
    if segment.type_ == hangups.schemas.SegmentType.TEXT:
        message.append(text)
    elif segment.type_ == hangups.schemas.SegmentType.LINK:
        message.append(
            '<a href="{}">{}</a>'.format(segment.link_target if segment.link_target else text, text)
        )
    elif segment.type_ == hangups.schemas.SegmentType.LINE_BREAK:
        message.append('<br>\n')
    else:
        logger.warning('Ignoring unknown chat message segment type: {}'.format(segment.type_))

    if not segment.type_ == hangups.schemas.SegmentType.LINE_BREAK:
        for is_f, f in ((segment.is_bold, 'b'), (segment.is_italic, 'i'),
                        (segment.is_strikethrough, 's'), (segment.is_underline, 'u')):
            if is_f:
                message.insert(0, '<{}>'.format(f))
                message.append('</{}>'.format(f))

    return ''.join(message)


def message_to_html(conv_event):
    """Create simple HTML from ChatMessageEvent"""
    lines = [segment_to_html(segment) for segment in conv_event.segments]
    attachments = ['<a href="{}">{}</a>'.format(url, url) for url in conv_event.attachments]

    html = ["".join(lines)]
    html.extend(attachments)

    return '<br>\n'.join(html)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'qhangups/qhangupsbrowser.ui'
#
# Created: Wed May 20 12:38:46 2015
#      by: PyQt4 UI code generator 4.11.3
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)

class Ui_QHangupsBrowser(object):
    def setupUi(self, QHangupsBrowser):
        QHangupsBrowser.setObjectName(_fromUtf8("QHangupsBrowser"))
        QHangupsBrowser.resize(600, 450)
        self.verticalLayout = QtGui.QVBoxLayout(QHangupsBrowser)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.browserWebView = QtWebKit.QWebView(QHangupsBrowser)
        self.browserWebView.setUrl(QtCore.QUrl(_fromUtf8("about:blank")))
        self.browserWebView.setObjectName(_fromUtf8("browserWebView"))
        self.verticalLayout.addWidget(self.browserWebView)

        self.retranslateUi(QHangupsBrowser)
        QtCore.QMetaObject.connectSlotsByName(QHangupsBrowser)

    def retranslateUi(self, QHangupsBrowser):
        QHangupsBrowser.setWindowTitle(_translate("QHangupsBrowser", "QHangups - Browser", None))

from PyQt4 import QtWebKit



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'qhangups/qhangupssettings.ui'
#
# Created: Wed May 20 12:38:46 2015
#      by: PyQt4 UI code generator 4.11.3
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)

class Ui_QHangupsSettings(object):
    def setupUi(self, QHangupsSettings):
        QHangupsSettings.setObjectName(_fromUtf8("QHangupsSettings"))
        QHangupsSettings.resize(370, 140)
        self.gridLayout = QtGui.QGridLayout(QHangupsSettings)
        self.gridLayout.setObjectName(_fromUtf8("gridLayout"))
        self.sendClientActiveCheckBox = QtGui.QCheckBox(QHangupsSettings)
        self.sendClientActiveCheckBox.setObjectName(_fromUtf8("sendClientActiveCheckBox"))
        self.gridLayout.addWidget(self.sendClientActiveCheckBox, 0, 0, 1, 2)
        self.sendReadStateCheckBox = QtGui.QCheckBox(QHangupsSettings)
        self.sendReadStateCheckBox.setObjectName(_fromUtf8("sendReadStateCheckBox"))
        self.gridLayout.addWidget(self.sendReadStateCheckBox, 1, 0, 1, 2)
        self.enterSendMessageCheckBox = QtGui.QCheckBox(QHangupsSettings)
        self.enterSendMessageCheckBox.setObjectName(_fromUtf8("enterSendMessageCheckBox"))
        self.gridLayout.addWidget(self.enterSendMessageCheckBox, 2, 0, 1, 2)
        self.label = QtGui.QLabel(QHangupsSettings)
        self.label.setObjectName(_fromUtf8("label"))
        self.gridLayout.addWidget(self.label, 3, 0, 1, 1)
        self.languageComboBox = QtGui.QComboBox(QHangupsSettings)
        self.languageComboBox.setObjectName(_fromUtf8("languageComboBox"))
        self.gridLayout.addWidget(self.languageComboBox, 3, 1, 1, 1)
        spacerItem = QtGui.QSpacerItem(47, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem, 3, 2, 1, 1)
        self.buttonBox = QtGui.QDialogButtonBox(QHangupsSettings)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtGui.QDialogButtonBox.Cancel|QtGui.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName(_fromUtf8("buttonBox"))
        self.gridLayout.addWidget(self.buttonBox, 4, 1, 1, 2)

        self.retranslateUi(QHangupsSettings)
        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL(_fromUtf8("accepted()")), QHangupsSettings.accept)
        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL(_fromUtf8("rejected()")), QHangupsSettings.reject)
        QtCore.QMetaObject.connectSlotsByName(QHangupsSettings)
        QHangupsSettings.setTabOrder(self.sendClientActiveCheckBox, self.sendReadStateCheckBox)
        QHangupsSettings.setTabOrder(self.sendReadStateCheckBox, self.enterSendMessageCheckBox)
        QHangupsSettings.setTabOrder(self.enterSendMessageCheckBox, self.languageComboBox)
        QHangupsSettings.setTabOrder(self.languageComboBox, self.buttonBox)

    def retranslateUi(self, QHangupsSettings):
        QHangupsSettings.setWindowTitle(_translate("QHangupsSettings", "QHangups - Settings", None))
        self.sendClientActiveCheckBox.setText(_translate("QHangupsSettings", "Send client active notifications", None))
        self.sendReadStateCheckBox.setText(_translate("QHangupsSettings", "Send read state notifications", None))
        self.enterSendMessageCheckBox.setText(_translate("QHangupsSettings", "Press Enter to send message (default is Ctrl+Enter)", None))
        self.label.setText(_translate("QHangupsSettings", "Language:", None))

